@@ -1513,8 +1513,7 @@ class Setup(_Admin, _Confirm, _Maintenance, Call):
         @classmethod
         def iexpand_arch(cls, arch):
             if arch == "native":
-                for native_arch in dist.Archs.native():
-                    yield native_arch
+                yield from dist.Archs.native()
             else:
                 yield arch
 
@@ -1676,16 +1675,17 @@ class Setup(_Admin, _Confirm, _Maintenance, Call):
                 return diff
 
         for codename in (dist.Codename(c) for c in setup["sources"]):
-            for source_codename in codename.setup_sources():
+            for source in [{"codename": codename.codename}] + codename.setup_extra_sources():
+                apt_keys = source.get("apt_keys", codename.setup.get("apt_keys", []))
                 SourceInstance(self,
                                util.models().Source,
-                               codename.setup_options(source_codename),
+                               source.get("options", {}),
                                {
-                                   "apt_keys": codename.setup_apt_keys(source_codename),
-                                   "apt_key_instances": [AptKeyInstance(self, util.models().AptKey, {}, {}, key_id=long_key_id) for long_key_id in codename.setup_apt_keys(source_codename)],
+                                   "apt_keys": apt_keys,
+                                   "apt_key_instances": [AptKeyInstance(self, util.models().AptKey, {}, {}, key_id=long_key_id) for long_key_id in apt_keys],
                                },
-                               origin=codename.setup_origin(source_codename),
-                               codename=source_codename)
+                               origin=source.get("origin", codename.origin),
+                               codename=source["codename"])
 
         # Distributions
         class DistributionInstance(Setup.Instance):
@@ -1745,13 +1745,13 @@ class Setup(_Admin, _Confirm, _Maintenance, Call):
                                                                                   util.models().PrioritySource,
                                                                                   {},
                                                                                   {},
-                                                                                  source=util.models().Source.mbd_get_or_none(codename=source_codename),
-                                                                                  priority=500 if codename.setup_is_security(source_codename) else 1)  # Smartly add prio source: Security gets prio=500, others prio=1 (opt-in).
-                                                                   for source_codename in (c for c in codename.setup_sources() if c != codename.codename)],
+                                                                                  source=util.models().Source.mbd_get_or_none(codename=extra_source["codename"], origin=extra_source.get("origin", codename.origin)),
+                                                                                  priority=extra_source.get("priority", 1))
+                                                                   for extra_source in codename.setup_extra_sources()],
                                      "component_instances": [Setup.Instance(self, util.models().Component, {}, {}, name=comp) for comp in codename.setup_components()],
                                      "archs": archs,
                                  },
-                                 base_source=util.models().Source.mbd_get_or_none(codename=codename.codename))
+                                 base_source=util.models().Source.mbd_get_or_none(codename=codename.codename, origin=codename.origin))
 
         # Repositories
         class LayoutInstance(Setup.Instance):
@@ -1806,7 +1806,7 @@ class Setup(_Admin, _Confirm, _Maintenance, Call):
                                getattr(util.models(), f"{setup['chroots_backend']}Chroot"),
                                codename.setup_chroot_options(),
                                {},
-                               source=util.models().Source.mbd_get_or_none(codename=codename.codename),
+                               source=util.models().Source.mbd_get_or_none(codename=codename.codename, origin=codename.origin),
                                architecture=Setup.Instance(self, util.models().Architecture, {}, {}, name=arch).obj)
 
         # Remotes
